@@ -171,12 +171,18 @@ namespace {
 
     } else {
       // search for block Do loop
-      pos = lowline.find("end",pos);
+      if(lowline.compare(pos, 3, "end") != 0)
+        return false;
+
+      pos = lowline.find_first_not_of(" \t", pos+3);
       if(pos==string::npos) return false;
 
-      pos = lowline.find("do",pos+3);
-      if(pos==string::npos) return false;
-      
+      if(lowline.compare(pos, 2, "do") != 0)
+        return false;
+
+      if(!(lowline[pos+2] == ' ' || lowline[pos+2] == '\t' || lowline[pos+2] == '\0'))
+        return false;
+
       // search for label
       if(toplabel.size()) {
 	
@@ -206,7 +212,7 @@ namespace {
     pragma->find_name();
     pragma->pline=save_pline;               // reset parse position
     pragma->ppos=save_ppos;
-    if(pragma->name.find("do ")!=string::npos) {
+    if(pragma->name.find("do")!=string::npos) {
       linetype=PRAGMA_LOOPSTART;
       if(pragma->name == "enddo")              linetype=PRAGMA_LOOPEND;
       else if(pragma->name == "paralleldo")    linetype=PRAGMA_PARLOOPSTART;
@@ -336,6 +342,10 @@ void process_fortran(istream& is, const char* infile, ostream& os,
     /* workaround for bogus getline implementations */
     if ( line.size() == 1 && line[0] == '\0' ) break;
 
+    /* remove possible trailing Carriage Return from line */
+    if ( line.size() > 0 && line[line.length()-1] == '\r' )
+      line.erase(line.length()-1);
+
     ++lineno;
     string lowline(line);
     transform(line.begin(), line.end(), lowline.begin(), fo_tolower());
