@@ -9,7 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2008 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2011 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2006-2009 Mellanox Technologies. All rights reserved.
  * Copyright (c) 2006-2007 Los Alamos National Security, LLC.  All rights
  *                         reserved.
@@ -166,6 +166,14 @@ int btl_openib_register_mca_params(void)
                   1, &ival, 0));
     mca_btl_openib_component.warn_nonexistent_if = (0 != ival);
 
+    /* If we print a warning about not having enough registered memory
+       available, do we want to abort? */
+    CHECK(reg_int("abort_not_enough_reg_mem", NULL,
+                  "If there is not enough registered memory available on the system for Open MPI to function properly, Open MPI will issue a warning.  If this MCA parameter is set to true, then Open MPI will also abort all MPI jobs "
+                  "(0 = warn, but do not abort; any other value = warn and abort)",
+                  0, &ival, 0));
+    mca_btl_openib_component.abort_not_enough_reg_mem = (0 != ival);
+
     if (OMPI_HAVE_IBV_FORK_INIT) {
         ival2 = -1;
     } else {
@@ -248,10 +256,11 @@ int btl_openib_register_mca_params(void)
                   REGINT_GE_ONE));
 
     CHECK(reg_int("cq_size", "ib_cq_size",
-                  "Size of the OpenFabrics completion "
-                  "queue (will automatically be set to a minimum of "
-                  "(2 * number_of_peers * btl_openib_rd_num))",
-                  1000, &ival, REGINT_GE_ONE));
+                  "Minimum size of the OpenFabrics completion queue "
+                  "(CQs are automatically sized based on the number "
+                  "of peer MPI processes; this value determines the "
+                  "*minimum* size of all CQs)",
+                  8192, &ival, REGINT_GE_ONE));
     mca_btl_openib_component.ib_cq_size[BTL_OPENIB_LP_CQ] =
         mca_btl_openib_component.ib_cq_size[BTL_OPENIB_HP_CQ] = (uint32_t) ival;
 
@@ -397,6 +406,15 @@ int btl_openib_register_mca_params(void)
     }
     mca_btl_openib_component.ib_service_level = (uint32_t) ival;
 
+#if (ENABLE_DYNAMIC_SL)
+    CHECK(reg_int("ib_path_record_service_level", NULL,
+                  "Enable getting InfiniBand service level from PathRecord "
+                  "(must be >= 0, 0 = disabled, positive = try to get the "
+                  "service level from PathRecord)",
+                  0, &ival, REGINT_GE_ZERO));
+    mca_btl_openib_component.ib_path_record_service_level = (uint32_t) ival;
+#endif
+
     CHECK(reg_int("use_eager_rdma", NULL, "Use RDMA for eager messages "
                   "(-1 = use device default, 0 = do not use eager RDMA, "
                   "1 = use eager RDMA)",
@@ -531,8 +549,9 @@ int btl_openib_register_mca_params(void)
 #if BTL_OPENIB_FAILOVER_ENABLED
     mca_btl_openib_module.super.btl_flags |= MCA_BTL_FLAGS_FAILOVER_SUPPORT;
 #endif
-    mca_btl_openib_module.super.btl_bandwidth = 800;
-    mca_btl_openib_module.super.btl_latency = 10;
+    /* Default to bandwidth auto-detection */
+    mca_btl_openib_module.super.btl_bandwidth = 0;
+    mca_btl_openib_module.super.btl_latency = 4;
     CHECK(mca_btl_base_param_register(
             &mca_btl_openib_component.super.btl_version,
             &mca_btl_openib_module.super));
@@ -593,6 +612,41 @@ int btl_openib_register_mca_params(void)
                      NULL, &mca_btl_openib_component.ipaddr_exclude,
                      0));
 
+    CHECK(reg_int("gid_index", NULL,
+                  "GID index to use on verbs device ports",
+                  0, &mca_btl_openib_component.gid_index,
+                  REGINT_GE_ZERO));
+
+#if BTL_OPENIB_MALLOC_HOOKS_ENABLED
+    CHECK(reg_int("memalign", NULL,
+                  "[64 | 32 | 0] - Enable (64bit or 32bit)/Disable(0) memory"
+                  "alignment for all malloc calls if btl openib is used.",
+                  32, &mca_btl_openib_component.use_memalign,
+                  REGINT_GE_ZERO));
+    
+    if (mca_btl_openib_component.use_memalign != 32  
+        && mca_btl_openib_component.use_memalign != 64
+        && mca_btl_openib_component.use_memalign != 0){ 
+        orte_show_help("help-mpi-btl-openib.txt", "invalid mca param value",
+                       true, "Wrong btl_openib_memalign parameter value. Allowed values: 64, 32, 0.",
+                       "btl_openib_memalign is reset to 32");
+        mca_btl_openib_component.use_memalign = 32; 
+    }
+    reg_int("memalign_threshold", NULL,
+            "Allocating memory more than btl_openib_memalign_threshhold"
+            "bytes will automatically be algined to the value of btl_openib_memalign bytes."
+            "memalign_threshhold defaults to the same value as mca_btl_openib_eager_limit.",
+            mca_btl_openib_component.eager_limit,
+            &ival,
+            REGINT_GE_ZERO);
+    if (ival < 0){
+        orte_show_help("help-mpi-btl-openib.txt", "invalid mca param value",
+                       true, "btl_openib_memalign_threshold must be positive",
+                       "btl_openib_memalign_threshold is reset to btl_openib_eager_limit");
+        ival = mca_btl_openib_component.eager_limit;
+    }
+    mca_btl_openib_component.memalign_threshold = (size_t)ival;
+#endif
     /* Register any MCA params for the connect pseudo-components */
     if (OMPI_SUCCESS == ret) {
         ret = ompi_btl_openib_connect_base_register();
