@@ -17,12 +17,18 @@
  */
 
 #include "orte_config.h"
+
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+
 #include "orte/constants.h"
 #include "orte/types.h"
 
 #include "opal/mca/mca.h"
 #include "opal/mca/base/base.h"
 #include "opal/class/opal_list.h"
+#include "opal/util/output.h"
 
 #include "orte/util/show_help.h"
 #include "opal/dss/dss.h"
@@ -33,6 +39,7 @@
 #include "orte/util/hostfile/hostfile.h"
 #include "orte/util/dash_host/dash_host.h"
 #include "orte/util/proc_info.h"
+#include "orte/util/comm/comm.h"
 
 #include "orte/mca/ras/base/ras_private.h"
 
@@ -83,7 +90,8 @@ int orte_ras_base_allocate(orte_job_t *jdata)
     orte_node_t *node;
     orte_std_cntr_t i;
     bool override_oversubscribed;
-    orte_app_context_t **apps;
+    orte_app_context_t *app;
+    bool default_hostfile_used;
 
     OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
                          "%s ras:base:allocate",
@@ -130,6 +138,13 @@ int orte_ras_base_allocate(orte_job_t *jdata)
     if (NULL != orte_ras_base.active_module)  {
         /* read the allocation */
         if (ORTE_SUCCESS != (rc = orte_ras_base.active_module->allocate(&nodes))) {
+            if (ORTE_ERR_SYSTEM_WILL_BOOTSTRAP == rc) {
+                /* this module indicates that nodes will be discovered
+                 * on a bootstrap basis, so all we do here is add our
+                 * own node to the list
+                 */
+                goto addlocal;
+            }
             ORTE_ERROR_LOG(rc);
             OBJ_DESTRUCT(&nodes);
             return rc;
@@ -146,6 +161,8 @@ int orte_ras_base_allocate(orte_job_t *jdata)
             return rc;
         }
         OBJ_DESTRUCT(&nodes);
+        /* flag that the allocation is managed */
+        orte_managed_allocation = true;
         goto DISPLAY;
     } else if (orte_allocation_required) {
         /* if nothing was found, and an allocation is
@@ -166,46 +183,9 @@ int orte_ras_base_allocate(orte_job_t *jdata)
     
     /* nothing was found, or no active module was alive. Our next
      * option is to look for a hostfile and assign our global
-     * pool from there. First, we check for a default hostfile
-     * as set by an mca param.
+     * pool from there.
      *
-     * Note that any relative node syntax found in the hostfile will
-     * generate an error in this scenario, so only non-relative syntax
-     * can be present
-     */
-    if (NULL != orte_default_hostfile) {
-        OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
-                             "%s ras:base:allocate parsing default hostfile %s",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             orte_default_hostfile));
-        
-        /* a default hostfile was provided - parse it */
-        if (ORTE_SUCCESS != (rc = orte_util_add_hostfile_nodes(&nodes,
-                                                               &override_oversubscribed,
-                                                               orte_default_hostfile))) {
-            ORTE_ERROR_LOG(rc);
-            OBJ_DESTRUCT(&nodes);
-            return rc;
-        }
-    }
-    /* if something was found in the default hostfile, we use that as our global
-     * pool - set it and we are done
-     */
-    if (!opal_list_is_empty(&nodes)) {
-        /* store the results in the global resource pool - this removes the
-         * list items
-         */
-        if (ORTE_SUCCESS != (rc = orte_ras_base_node_insert(&nodes, jdata))) {
-            ORTE_ERROR_LOG(rc);
-        }
-        /* update the jdata object with override_oversubscribed flag */
-        jdata->oversubscribe_override = override_oversubscribed;
-        /* cleanup */
-        OBJ_DESTRUCT(&nodes);
-        goto DISPLAY;
-    }
-    
-    /* Individual hostfile names, if given, are included
+     * Individual hostfile names, if given, are included
      * in the app_contexts for this job. We therefore need to
      * retrieve the app_contexts for the job, and then cycle
      * through them to see if anything is there. The parser will
@@ -213,33 +193,72 @@ int orte_ras_base_allocate(orte_job_t *jdata)
      * the resulting list contains the UNION of all nodes specified
      * in hostfiles from across all app_contexts
      *
+     * Any app that has no hostfile but has a dash-host, will have
+     * those nodes added to the list
+     *
+     * Any app that fails to have a hostfile or a dash-host will be given the
+     * default hostfile, if we have it
+     *
      * Note that any relative node syntax found in the hostfiles will
      * generate an error in this scenario, so only non-relative syntax
      * can be present
      */
-    
-    /* convenience def */
-    apps = (orte_app_context_t**)jdata->apps->addr;
-    
-    for (i=0; i < jdata->num_apps; i++) {
-        if (NULL != apps[i]->hostfile) {
-            
+    default_hostfile_used = false;
+    for (i=0; i < jdata->apps->size; i++) {
+        if (NULL == (app = (orte_app_context_t*)opal_pointer_array_get_item(jdata->apps, i))) {
+            continue;
+        }
+        if (NULL != app->hostfile) {
             OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
-                                 "%s ras:base:allocate checking hostfile %s",
+                                 "%s ras:base:allocate adding hostfile %s",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                                 apps[i]->hostfile));
+                                 app->hostfile));
             
             /* hostfile was specified - parse it and add it to the list */
             if (ORTE_SUCCESS != (rc = orte_util_add_hostfile_nodes(&nodes,
-                                                &override_oversubscribed,
-                                                apps[i]->hostfile))) {
+                                                                   &override_oversubscribed,
+                                                                   app->hostfile))) {
                 ORTE_ERROR_LOG(rc);
                 OBJ_DESTRUCT(&nodes);
                 return rc;
             }
+        } else if (NULL != app->dash_host) {
+            OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
+                                 "%s ras:base:allocate adding dash_hosts",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
+            if (ORTE_SUCCESS != (rc = orte_util_add_dash_host_nodes(&nodes,
+                                                                    &override_oversubscribed,
+                                                                    app->dash_host))) {
+                ORTE_ERROR_LOG(rc);
+                OBJ_DESTRUCT(&nodes);
+                return rc;
+            }
+        } else if (!default_hostfile_used) {
+            if (NULL != orte_default_hostfile) {
+                OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
+                                     "%s ras:base:allocate parsing default hostfile %s",
+                                     ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                     orte_default_hostfile));
+        
+                /* a default hostfile was provided - parse it */
+                if (ORTE_SUCCESS != (rc = orte_util_add_hostfile_nodes(&nodes,
+                                                                       &override_oversubscribed,
+                                                                       orte_default_hostfile))) {
+                    ORTE_ERROR_LOG(rc);
+                    OBJ_DESTRUCT(&nodes);
+                    return rc;
+                }
+            }
+            /* only look at it once */
+            default_hostfile_used = true;
+        }
+        /* update the jdata object with override_oversubscribed flag, but don't
+         * overwrite a prior setting to true
+         */
+        if (!jdata->oversubscribe_override) {
+            jdata->oversubscribe_override = override_oversubscribed;
         }
     }
-
     /* if something was found in the hostfile(s), we use that as our global
      * pool - set it and we are done
      */
@@ -249,49 +268,32 @@ int orte_ras_base_allocate(orte_job_t *jdata)
          */
         if (ORTE_SUCCESS != (rc = orte_ras_base_node_insert(&nodes, jdata))) {
             ORTE_ERROR_LOG(rc);
+            OBJ_DESTRUCT(&nodes);
+            return rc;
         }
-        /* update the jdata object with override_oversubscribed flag */
-        jdata->oversubscribe_override = override_oversubscribed;
         /* cleanup */
         OBJ_DESTRUCT(&nodes);
         goto DISPLAY;
     }
-    
-    
-    
+
     OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
-                         "%s ras:base:allocate nothing found in hostfiles - checking dash-host options",
+                         "%s ras:base:allocate nothing found in hostfiles or dash-host - checking for rankfile",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
     
-    /* Our next option is to look for hosts provided via the -host
-     * command line option. If they are present, we declare this
-     * to represent not just a mapping, but to define the global
-     * resource pool in the absence of any other info.
-     *
-     * -host lists are provided as part of the app_contexts for
-     * this job. We therefore need to retrieve the app_contexts
-     * for the job, and then cycle through them to see if anything
-     * is there. The parser will add the -host nodes to our list - i.e.,
-     * the resulting list contains the UNION of all nodes specified
-     * by -host across all app_contexts
-     *
-     * Note that any relative node syntax found in the -host lists will
-     * generate an error in this scenario, so only non-relative syntax
-     * can be present
+    /* Our next option is to look for a rankfile - if one was provided, we
+     * will use its nodes to create a default allocation pool
      */
-    for (i=0; i < jdata->num_apps; i++) {
-        if (NULL != apps[i]->dash_host) {
-            if (ORTE_SUCCESS != (rc = orte_util_add_dash_host_nodes(&nodes,
-                                                    &override_oversubscribed,
-                                                    apps[i]->dash_host))) {
-                ORTE_ERROR_LOG(rc);
-                OBJ_DESTRUCT(&nodes);
-                return rc;
-            }
+    if (NULL != orte_rankfile) {
+        /* check the rankfile for node information */
+        if (ORTE_SUCCESS != (rc = orte_util_add_hostfile_nodes(&nodes,
+                                                               &override_oversubscribed,
+                                                               orte_rankfile))) {
+            ORTE_ERROR_LOG(rc);
+            OBJ_DESTRUCT(&nodes);
+            return rc;
         }
     }
-
-    /* if something was found in -host, we use that as our global
+    /* if something was found in rankfile, we use that as our global
      * pool - set it and we are done
      */
     if (!opal_list_is_empty(&nodes)) {
@@ -302,18 +304,18 @@ int orte_ras_base_allocate(orte_job_t *jdata)
             ORTE_ERROR_LOG(rc);
         }
         /* update the jdata object with override_oversubscribed flag */
-        jdata->oversubscribe_override = override_oversubscribed;
+        jdata->oversubscribe_override = false;
         /* cleanup */
         OBJ_DESTRUCT(&nodes);
         goto DISPLAY;
     }
     
     
-    
     OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
-                         "%s ras:base:allocate nothing found in dash-host - inserting current node",
+                         "%s ras:base:allocate nothing found in rankfile - inserting current node",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
     
+addlocal:
     /* if nothing was found by any of the above methods, then we have no
      * earthly idea what to do - so just add the local host
      */
@@ -346,6 +348,12 @@ int orte_ras_base_allocate(orte_job_t *jdata)
     OBJ_DESTRUCT(&nodes);
 
 DISPLAY:
+    /* are we to report this event? */
+    if (orte_report_events) {
+        if (ORTE_SUCCESS != (rc = orte_util_comm_report_event(ORTE_COMM_EVENT_ALLOCATE))) {
+            ORTE_ERROR_LOG(rc);
+        }
+    }
     /* shall we display the results? */
     if (orte_ras_base.display_alloc) {
         display_alloc();
@@ -353,3 +361,94 @@ DISPLAY:
     
     return rc;
 }
+
+int orte_ras_base_add_hosts(orte_job_t *jdata)
+{
+    int rc;
+    opal_list_t nodes;
+    bool override_oversubscribed;
+    int i;
+    orte_app_context_t *app;
+
+    /* construct a list to hold the results */
+    OBJ_CONSTRUCT(&nodes, opal_list_t);
+    
+    /* Individual add-hostfile names, if given, are included
+     * in the app_contexts for this job. We therefore need to
+     * retrieve the app_contexts for the job, and then cycle
+     * through them to see if anything is there. The parser will
+     * add the nodes found in each add-hostfile to our list - i.e.,
+     * the resulting list contains the UNION of all nodes specified
+     * in add-hostfiles from across all app_contexts
+     *
+     * Note that any relative node syntax found in the add-hostfiles will
+     * generate an error in this scenario, so only non-relative syntax
+     * can be present
+     */
+    
+    for (i=0; i < jdata->apps->size; i++) {
+        if (NULL == (app = (orte_app_context_t*)opal_pointer_array_get_item(jdata->apps, i))) {
+            continue;
+        }
+        if (NULL != app->add_hostfile) {
+            OPAL_OUTPUT_VERBOSE((5, orte_ras_base.ras_output,
+                                 "%s ras:base:add_hosts checking add-hostfile %s",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                 app->add_hostfile));
+            
+            /* hostfile was specified - parse it and add it to the list */
+            if (ORTE_SUCCESS != (rc = orte_util_add_hostfile_nodes(&nodes,
+                                                                   &override_oversubscribed,
+                                                                   app->add_hostfile))) {
+                ORTE_ERROR_LOG(rc);
+                OBJ_DESTRUCT(&nodes);
+                return rc;
+            }
+        }
+    }
+
+    /* We next check for and add any add-host options. Note this is
+     * a -little- different than dash-host in that (a) we add these
+     * nodes to the global pool regardless of what may already be there,
+     * and (b) as a result, any job and/or app_context can access them.
+     *
+     * Note that any relative node syntax found in the add-host lists will
+     * generate an error in this scenario, so only non-relative syntax
+     * can be present
+     */
+    for (i=0; i < jdata->apps->size; i++) {
+        if (NULL == (app = (orte_app_context_t*)opal_pointer_array_get_item(jdata->apps, i))) {
+            continue;
+        }
+        if (NULL != app->add_host) {
+            if (ORTE_SUCCESS != (rc = orte_util_add_dash_host_nodes(&nodes,
+                                                                    &override_oversubscribed,
+                                                                    app->add_host))) {
+                ORTE_ERROR_LOG(rc);
+                OBJ_DESTRUCT(&nodes);
+                return rc;
+            }
+        }
+    }
+    
+    /* if something was found, we add that to our global pool */
+    if (!opal_list_is_empty(&nodes)) {
+        /* store the results in the global resource pool - this removes the
+         * list items
+         */
+        if (ORTE_SUCCESS != (rc = orte_ras_base_node_insert(&nodes, jdata))) {
+            ORTE_ERROR_LOG(rc);
+        }
+        /* update the jdata object with override_oversubscribed flag */
+        jdata->oversubscribe_override = override_oversubscribed;
+        /* cleanup */
+        OBJ_DESTRUCT(&nodes);
+    }
+    
+    /* shall we display the results? */
+    if (orte_ras_base.display_alloc) {
+        display_alloc();
+    }
+    
+    return ORTE_SUCCESS;
+}
